import os
from optparse import OptionParser
import numpy as np
from ase.lattice import bulk
from ase.parallel import paropen
from ase.io.trajectory import Trajectory
from gpaw import GPAW, PW
from gpaw.upf import UPFSetupData
from gpaw.mpi import world
from gpaw.setup_data import SetupData

p = OptionParser()
p.add_option('--sym', default='Ag')
p.add_option('--type', default='paw')
p.add_option('--xc', default='PBE')
p.add_option('--force', action='store_true',
             help='overwrite if necessary')
p.add_option('--writetraj', action='store_true',
             help='write geometries to trajectory file and nothing else')
opts, args = p.parse_args()

sym = opts.sym
type = opts.type

if type == 'upf':
    s = 'sg15'
#    s = UPFSetupData('%s_ONCV_PBE-1.0.upf' % sym)
elif type == 'paw':
    s = 'paw'
elif type == 'hgh':
    s = UPFSetupData('%s.pbe-d-hgh.UPF' % sym)
    #from gpaw.hgh import setups, sc_setups
    #if sym in sc_setups:
    #    s = 'hgh.sc'
    #else:
    #    s = 'hgh'
elif type == 'paw19':
    from gpaw import setup_paths
    setup_paths.insert(0, '.')
    if opts.xc == 'PBE' or 'grrr' in opts.xc:
        grrr = 'PBE'
    elif 'vdW-DF' in opts.xc:
        grrr = 'revPBE'
    else:
        raise ValueError('something with paw19')
    s = SetupData(sym, grrr, '19')
else:
    raise ValueError('unknown type %s' % type)

a0data = dict(Ag=4.09,
              Cu=3.61,
              Au=4.08)

#a0 = a0data[sym]


fileid = '%s.%s.%s' % (sym, type, opts.xc)


fname = 'results.%s.txt' % fileid
world.barrier()
if not opts.writetraj:
    if os.path.exists(fname) and not opts.force:
        raise IOError('File exists')
    fd = paropen(fname, 'w')

parallel = dict(kpt=world.size)

xc = opts.xc
from vdw_ours import VDW
if opts.xc == 'vdW-DF-ours':
    xc = VDW()
if opts.xc == 'vdW-DF-ours-conj':
    xc = VDW()
if opts.xc == 'vdW-DF-ours-grrr':
    xc = VDW('PBE')
if opts.xc == 'libvdwxc':
    from gpaw.xc.libvdwxc import VDWDF
    xc = VDWDF()
if opts.xc == 'libvdwxc-DF2':
    from gpaw.xc.libvdwxc import VDWDF2
    xc = VDWDF2()
if opts.xc == 'libvdwxc-CX':
    from gpaw.xc.libvdwxc import VDWDFCX
    xc = VDWDFCX()
    #parallel=dict(kpt=4, domain=4)

pwcutoff = 1200.0
nkpts = 16
ngpts = 24
if sym in 'Si C He Ne Ar'.split():
    pwcutoff = 1000.0
    nkpts = 4
    ngpts = 36
if sym in 'Si C'.split():
    nkpts = 8
if sym == 'C':
    ngpts = 24

if opts.xc == 'vdW-DF-PBE':
    xc = 'vdW-DF' # XXX


calc = GPAW(mode=PW(pwcutoff),
            setups={0: s},
            xc=xc,
            nbands=-5,
            parallel=parallel,
            gpts=(ngpts, ngpts, ngpts),
            kpts=(nkpts, nkpts, nkpts),
            txt='gpaw.%s.txt' % fileid)

def is_vdw_df(xc):
    return xc.startswith('vdW-DF') or xc.startswith('libvdwxc')

def get_cx_values(sym):
    a0_exp = dict(Cu=3.61,
                  Ag=4.08,
                  Au=4.09)[sym]
    astart = a0_exp * 0.98
    astop = a0_exp * 1.02
    a0values = np.linspace(astart, astop, 12)
    return a0values
                  

def get_a0_values():
    
    v = ValueError('No refdata')

    xc0 = opts.xc

    try:
        if 'cx' in xc0.lower():
            return get_cx_values(opts.sym)
    except KeyError:
        pass

    if sym == 'Ag':
        if xc0 in ['PBE', 'revPBE']:
            return np.linspace(4.088, 4.2, 8)
        elif 'CX' in xc0:
            a0 = 3.595
        elif is_vdw_df(xc0):
            return np.linspace(4.19, 4.36, 8)
        else:
            raise v
    elif sym == 'Cu':
        if xc0 in ['PBE', 'revPBE']:
            return np.linspace(3.581, 3.691, 8)
        elif is_vdw_df(xc0):
            return np.linspace(3.6198, 3.8239, 10)
        else:
            raise v
    elif sym == 'Au':
        if xc0 in ['PBE', 'revPBE']:
            return np.linspace(4.12, 4.23, 8)
        elif is_vdw_df(xc0):
            return np.linspace(4.185, 4.35, 10)
        else:
            raise v
    elif sym == 'He':
        if xc0 in ['PBE', 'revPBE']:
            return np.linspace(4.05, 4.25, 8)
        elif is_vdw_df(xc0):
            return np.linspace(3.85, 4.1, 10)
        else:
            raise v
    elif sym == 'Ne':
        if xc0 in ['PBE', 'revPBE']:
            return np.linspace(4.5, 4.8, 8)
        elif is_vdw_df(xc0):
            return np.linspace(4.15, 4.45, 10)
        else:
            raise v
    elif sym == 'Ar':
        if xc0 in ['PBE', 'revPBE']:
            return np.linspace(5.8, 6.05, 8)
        elif is_vdw_df(xc0):
            return np.linspace(5.4, 5.6, 8)
        else:
            raise v
    elif sym == 'Ni':
        if xc0 in ['PBE', 'revPBE']:
            return np.linspace(3.45, 3.6, 10)
        elif is_vdw_df(xc0):
            return np.linspace(3.5, 3.7, 10)
        else:
            raise v
    elif sym == 'Zn':
        if xc0 in ['PBE', 'revPBE']:
            return np.linspace(3.85, 4.05, 10)
        elif is_vdw_df(xc0):
            return np.linspace(4.0, 4.35, 10)
        else:
            raise v
    elif sym == 'C':
        a0 = 3.57
        return np.linspace(a0 - 0.4, a0 + 0.6, 24)
    elif sym == 'Si':
        a0 = 5.43
        return np.linspace(a0 - 0.5, a0 + 0.8, 32)
    else:
        raise v

if opts.writetraj:
    fname = 'geom.%s.%s.traj' % (sym, opts.xc)
    traj = Trajectory(fname, 'w')

#cell0 = system.get_cell()
for i, a0 in enumerate(get_a0_values()):
    if 'DF2' in opts.xc:
        a0 *= 1.015
    if sym in ['C', 'Si']:
        system = bulk(sym, 'diamond', a0)
    else:
        system = bulk(sym, 'fcc', a0)
    if opts.writetraj:
        traj.write(system)
        continue
    system.set_calculator(calc)
    scale = 0.0
    #a = a0 * (1.0 + scale)
    #system.cell = cell0 * (1.0 + scale)
    e = system.get_potential_energy()
    print >> fd, '%d %f %f %f' % (i, a0, scale, e)
    fd.flush()
